//! Auto-generated constants.
//!
//! This module contains constants that are automatically generated:
//! - `calendar_ids` and `locale_ids`: generated by `build.rs`
//! - `schemas`: generated by `cargo run --bin generate-schema --features schema-gen`
//!
//! Do not modify these files manually - they will be overwritten on regeneration.

pub mod calendar_ids;
pub mod locale_ids;
pub mod schemas;

pub use calendar_ids::*;
pub use locale_ids::*;
